from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app.api.deps import get_current_active_user, get_db
from app.crud.crud_tag import (
    create_tag,
    delete_tag,
    get_tag,
    get_tag_by_name,
    get_tags,
    update_tag,
)
from app.models.user import User
from app.schemas.tag import Tag, TagCreate, TagUpdate

router = APIRouter()


@router.get("/", response_model=List[Tag])
def read_tags(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Retrieve tags.
    """
    tags = get_tags(db=db, owner_id=current_user.id, skip=skip, limit=limit)
    return tags


@router.post("/", response_model=Tag)
def create_tag_item(
    *,
    db: Session = Depends(get_db),
    tag_in: TagCreate,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Create new tag.
    """
    tag = get_tag_by_name(db=db, name=tag_in.name, owner_id=current_user.id)
    if tag:
        raise HTTPException(
            status_code=400,
            detail="The tag with this name already exists",
        )
    tag = create_tag(db=db, tag_in=tag_in, owner_id=current_user.id)
    return tag


@router.get("/{id}", response_model=Tag)
def read_tag(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Get tag by ID.
    """
    tag = get_tag(db=db, tag_id=id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    if tag.owner_id != current_user.id:
        raise HTTPException(status_code=400, detail="Not enough permissions")
    return tag


@router.put("/{id}", response_model=Tag)
def update_tag_item(
    *,
    db: Session = Depends(get_db),
    id: int,
    tag_in: TagUpdate,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Update a tag.
    """
    tag = get_tag(db=db, tag_id=id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    if tag.owner_id != current_user.id:
        raise HTTPException(status_code=400, detail="Not enough permissions")
    tag = update_tag(db=db, db_obj=tag, obj_in=tag_in)
    return tag


@router.delete("/{id}", response_model=None, status_code=status.HTTP_204_NO_CONTENT)
def delete_tag_item(
    *,
    db: Session = Depends(get_db),
    id: int,
    current_user: User = Depends(get_current_active_user),
) -> Any:
    """
    Delete a tag.
    """
    tag = get_tag(db=db, tag_id=id)
    if not tag:
        raise HTTPException(status_code=404, detail="Tag not found")
    if tag.owner_id != current_user.id:
        raise HTTPException(status_code=400, detail="Not enough permissions")
    delete_tag(db=db, tag_id=id)
    return None